#include <cstdlib>
#include <iomanip>
#include <iostream>
#include "math.h"

#include <cuda_runtime.h>

#include "heat_equation.h"


int main()
{
    //  : a <= x <= b, c <= y <= d.
    const float a = 0;
    const float b = 1;
    const float c = 2;
    const float d = 3;

    //      x  y .
    const int nx = 500;
    const int ny = 500;

    //   .
    float dx = (b - a) / (float)(nx - 1);
    float dy = (d - c) / (float)(ny - 1);

    //       .
    float dt = 1e-7f;
    int num_time_steps = 100;

    //         CPU  GPU.
    //        .
	float * v = new float[nx * ny];
    float * v_gpu;
    cudaMalloc((void **) &v_gpu, nx * ny * sizeof(float));
    float * newV = new float[nx * ny];
    float * newV_gpu;
    cudaMalloc((void **) &newV_gpu, nx * ny * sizeof(float));
    float * v_verify = new float[nx * ny]; //   

    //   .
    for (int i = 0; i < nx; ++i)
        for (int j = 0; j < ny; ++j)
        {
            v[i * ny + j] = 0.0f;
            newV[i * ny + j] = 0.0f;
        }

    //     GPU.
    cudaMemcpy(v_gpu, v, nx * ny * sizeof(float), cudaMemcpyHostToDevice);
    cudaMemcpy(newV_gpu, newV, nx * ny * sizeof(float), cudaMemcpyHostToDevice);

    //  CPU-.
    for (int i = 0; i < num_time_steps; ++i)
    {
        //    .
        step(nx, ny, dx, dy, dt, a, c, v, newV);

        //   v  newV.
        float * tmp = v;
        v = newV;
        newV = tmp;
    }

    //  GPU-.
    for (int i = 0; i < num_time_steps; ++i)
    {
        //    .
        step_gpu(nx, ny, dx, dy, dt, a, c, v_gpu, newV_gpu);
        // ,    .
        cudaThreadSynchronize();

        //   v_gpu  newV_gpu.
        float * tmp = v_gpu;
        v_gpu = newV_gpu;
        newV_gpu = tmp;
    }

    //        .
    cudaMemcpy(v_verify, v_gpu, nx * ny * sizeof(float), cudaMemcpyDeviceToHost);

    //     v  v_verify.
    float difference = 0;
    for (int idx = 0; idx < nx * ny; ++idx)
        difference += (v[idx] - v_verify[idx]) * (v[idx] - v_verify[idx]);
    difference = sqrtf(difference);

    //  ,     .
    const float eps = 1e-5f;
    if (difference < eps)
        std::cout << "Test PASSED.\n";
    else
        std::cout << "Test FAILED.\n";

    //  .
    cudaFree(v_gpu);
    delete [] v_verify;
    delete [] v;
    cudaFree(newV_gpu);
    delete [] newV;

    return 0;
}
